/*
 * This code was created by Jeff Molofee '99 
 * (ported to Linux/SDL by Ti Leggett '01)
 *
 * If you've found this code useful, please let me know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 *
 * or for port-specific comments, questions, bugreports etc. 
 * email to leggett@eecs.tulane.edu
 */

#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_opengles.h>

/* Include partial OpenGL ES 1.0 GLU port header */
#define __USE_SDL_GLES__
#include "glues.h"

/* screen width, height, and bit depth */
#define WINDOW_WIDTH  320
#define WINDOW_HEIGHT 240
#define WINDOW_BPP     16
#define DATA_PATH "/Didj/ProgramFiles/lesson07/data/"

/* Whether or not lighting is on */
SDL_bool light=SDL_FALSE;

/* Array for all available keys */
SDL_bool pressedkeys[SDL_NUM_SCANCODES];

GLfloat xrot;         /* X Rotation */
GLfloat yrot;         /* Y Rotation */
GLfloat xspeed=0.25f; /* X Rotation Speed */
GLfloat yspeed=0.25f; /* Y Rotation Speed */
GLfloat z=-5.0f;      /* Depth Into The Screen */

/* Ambient Light Values (NEW) */
GLfloat LightAmbient[]={0.1f, 0.1f, 0.1f, 1.0f};
/* Diffuse Light Values (NEW) */
GLfloat LightDiffuse[]={1.0f, 1.0f, 1.0f, 1.0f};
/* Light Position (NEW) */
GLfloat LightPosition[]={0.0f, 0.0f, 2.0f, 1.0f};

/* Default OpenGL ES material settings */
GLfloat box_material_amb[4]={0.2f, 0.2f, 0.2f, 0.2f};
GLfloat box_material_dif[4]={0.8f, 0.8f, 0.8f, 0.8f};

GLuint filter=0;   /* Which Filter To Use */
GLuint texture[3]; /* Storage for 3 textures */

/* SDL window */
SDL_WindowID window;
/* SDL GL ES context, binded to SDL window */
SDL_GLContext glcontext=NULL;

/* function to release/destroy our resources and restoring the old desktop */
void DestroyApp(int returnCode)
{
    /* clean up the window */
    SDL_GL_DeleteContext(glcontext);
    SDL_DestroyWindow(window);

    SDL_Quit();

    /* and exit appropriately */
    exit(returnCode);
}

/* function to load in bitmap as a GL texture */
int LoadGLTextures()
{
    /* Status indicator */
    SDL_bool status=SDL_FALSE;

    /* Create storage space for the texture */
    SDL_Surface* TextureImage[1]; 

	printf("%s\n", DATA_PATH "crate.bmp");

    /* Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit */
    if ((TextureImage[0]=SDL_LoadBMP(DATA_PATH "crate.bmp")))
    {
        /* Since OpenGL ES 1.0/1.1 has no BGRA extension we must swap R & B */
        {
            GLuint it;
            GLuint jt;

            for (jt=0; jt<TextureImage[0]->h; jt++)
            {
                for (it=0; it<TextureImage[0]->w; it++)
                {
                    switch (TextureImage[0]->format->BytesPerPixel)
                    {
                        case 3: /* 24 bpp */
                             {
                                 unsigned char rvalue;
                                 unsigned char bvalue;

                                 rvalue=*((unsigned char*)TextureImage[0]->pixels+
                                     jt*TextureImage[0]->pitch+it*TextureImage[0]->format->BytesPerPixel);
                                 bvalue=*((unsigned char*)TextureImage[0]->pixels+
                                     jt*TextureImage[0]->pitch+it*TextureImage[0]->format->BytesPerPixel+2);
                                 /* Convert RGB to BGR and vice versa */
                                 *((unsigned char*)TextureImage[0]->pixels+
                                     jt*TextureImage[0]->pitch+it*TextureImage[0]->format->BytesPerPixel)=bvalue;
                                 *((unsigned char*)TextureImage[0]->pixels+
                                     jt*TextureImage[0]->pitch+it*TextureImage[0]->format->BytesPerPixel+2)=rvalue;
                             }
                             break;
                        case 4: /* 32 bpp */
                             {
                                 unsigned long rgbvalue;

                                 rgbvalue=*(unsigned long*)((unsigned char*)TextureImage[0]->pixels+
                                     jt*TextureImage[0]->pitch+it*TextureImage[0]->format->BytesPerPixel);
                                 /* Convert ARGB to ABGR and vice versa */
                                 rgbvalue=(rgbvalue & 0xFF00FF00) | ((rgbvalue<<16) & 0x00FF0000) |
                                          ((rgbvalue>>16) & 0x000000FF);
                                 *(unsigned long*)((unsigned char*)TextureImage[0]->pixels+
                                     jt*TextureImage[0]->pitch+it*TextureImage[0]->format->BytesPerPixel)=rgbvalue;
                             }
                             break;
                    }
                }
            }
        }

        /* Set the status to true */
        status=SDL_TRUE;

        /* Create The Textures */
        glGenTextures(3, &texture[0]);

        /* Load in texture 1 */
        /* Typical Texture Generation Using Data From The Bitmap */
        glBindTexture(GL_TEXTURE_2D, texture[0]);

        /* Generate The Texture */
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, TextureImage[0]->w,
                     TextureImage[0]->h, 0, GL_RGB,
                     GL_UNSIGNED_BYTE, TextureImage[0]->pixels);

        /* Nearest Filtering */
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

        /* Load in texture 2 */
        /* Typical Texture Generation Using Data From The Bitmap */
        glBindTexture(GL_TEXTURE_2D, texture[1]);

        /* Linear Filtering */
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

        /* Generate The Texture */
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, TextureImage[0]->w,
                     TextureImage[0]->h, 0, GL_RGB,
                     GL_UNSIGNED_BYTE, TextureImage[0]->pixels);

        /* Load in texture 3 using mipmaps */
        glBindTexture(GL_TEXTURE_2D, texture[2]);
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
        gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB,
                          TextureImage[0]->w, TextureImage[0]->h,
                          GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->pixels);
    }

    /* Free up any memory we may have used */
    if (TextureImage[0])
    {
        SDL_FreeSurface(TextureImage[0]);
    }

    return status;
}

/* function to reset our viewport after a window resize */
int resizeWindow(int width, int height)
{
    /* Height / width ration */
    GLfloat ratio;

    /* Protect against a divide by zero */
    if (height==0)
    {
        height=1;
    }

    ratio=(GLfloat)width/(GLfloat)height;

    /* Setup our viewport */
    glViewport(0, 0, (GLint)width, (GLint)height);

    /* change to the projection matrix and set our viewing volume. */
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    /* Set our perspective */
    gluPerspective(45.0f, ratio, 0.1f, 100.0f);

    /* Make sure we're chaning the model view and not the projection */
    glMatrixMode(GL_MODELVIEW);

    /* Reset The View */
    glLoadIdentity();

    return SDL_TRUE;
}

/* function to handle key press events */
void handleKeyPress(SDL_keysym* keysym, SDL_bool pressed)
{
    if (pressed==SDL_TRUE)
    {
       pressedkeys[keysym->scancode]=SDL_TRUE;
    }
    else
    {
       pressedkeys[keysym->scancode]=SDL_FALSE;
    }

    /* Handle key presses only for these keys */
    if ((pressedkeys[SDL_SCANCODE_ESCAPE]) && (pressed))
    {
        DestroyApp(0);
    }
    if ((pressedkeys[SDL_SCANCODE_F]) && (pressed))
    {
        filter=(filter+1) % 3;
    }
    if ((pressedkeys[SDL_SCANCODE_L]) && (pressed))
    {
        light=!light;
        if (!light)
        {
            glDisable(GL_LIGHTING);
        }
        else
        {
            glEnable(GL_LIGHTING);
        }
    }
}

void handleRepeatedKeys()
{
    /* Use autorepeat emulation for these keys */
    if (pressedkeys[SDL_SCANCODE_PAGEUP])
    {
        z-=0.02f;
    }
    if (pressedkeys[SDL_SCANCODE_PAGEDOWN])
    {
        z+=0.02f;
    }
    if (pressedkeys[SDL_SCANCODE_UP])
    {
        xspeed-=0.01f;
    }
    if (pressedkeys[SDL_SCANCODE_DOWN])
    {
        xspeed+=0.01f;
    }
    if (pressedkeys[SDL_SCANCODE_RIGHT])
    {
        yspeed-=0.01f;
    }
    if (pressedkeys[SDL_SCANCODE_LEFT])
    {
        yspeed+=0.01f;
    }
}

/* general OpenGL initialization function */
SDL_bool initGL(GLvoid)
{
    GLenum error;

    /* Clear error if it is */
    error=glGetError();

    /* Load in the texture */
    if (!LoadGLTextures())
    {
        return SDL_FALSE;
    }

    /* Enable Texture Mapping (NEW) */
    glEnable(GL_TEXTURE_2D);

    /* Enable smooth shading */
    glShadeModel(GL_SMOOTH);

    /* Set the background black */
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

    /* Depth buffer setup */
    glClearDepthf(1.0f);

    /* Enables Depth Testing */
    glEnable(GL_DEPTH_TEST);

    /* The Type Of Depth Test To Do */
    glDepthFunc(GL_LEQUAL);

    /* Really Nice Perspective Calculations */
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    /* Setup The Ambient Light */
    glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);

    /* Setup The Diffuse Light */
    glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);

    /* Position The Light */
    glLightfv(GL_LIGHT1, GL_POSITION, LightPosition);

    /* Enable Light */
    glEnable(GL_LIGHT1);

    /* Enable/Disable lighting */
    if (!light)
    {
        glDisable(GL_LIGHTING);
    }
    else
    {
        glEnable(GL_LIGHTING);
    }

    error=glGetError();
    if (error!=GL_NO_ERROR)
    {
       printf("OpenGL ES error: %08X\n", error);
    }

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, box_material_amb);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, box_material_dif);

    return SDL_TRUE;
}

/* Here goes our drawing code */
int drawGLScene(GLvoid)
{
    /* These are to calculate our fps */
    static GLint T0=0;
    static GLint Frames=0;
	static int loop=0;
    GLenum  error;
    GLfloat texcoords[4][2];
    GLfloat vertices[4][3];
    GLfloat normals[4][3];
    GLubyte indices[4]={0, 1, 3, 2}; /* QUAD to TRIANGLE_STRIP conversion; */

    /* Clear error if it is */
    error=glGetError();

    /* Clear The Screen And The Depth Buffer */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* Reset the view */
    glLoadIdentity();

    /* Translate Into/Out Of The Screen By z */
    glTranslatef(0.0f, 0.0f, z);

    glRotatef(xrot, 1.0f, 0.0f, 0.0f); /* Rotate On The X Axis By xrot */
    glRotatef(yrot, 0.0f, 1.0f, 0.0f); /* Rotate On The Y Axis By yrot */

    /* Select A Texture Based On filter */
    glBindTexture(GL_TEXTURE_2D, texture[filter]);

    /* Set pointers to vertices and texcoords */
    glVertexPointer(3, GL_FLOAT, 0, vertices);
    glTexCoordPointer(2, GL_FLOAT, 0, texcoords);
    glNormalPointer(GL_FLOAT, 0, normals);

    /* Enable vertices and texcoords arrays */
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);

    /* Front Face */
    /* Normal Pointing Towards Viewer */
    normals[0][0]=normals[1][0]=normals[2][0]=normals[3][0]=0.0f;
    normals[0][1]=normals[1][1]=normals[2][1]=normals[3][1]=0.0f;
    normals[0][2]=normals[1][2]=normals[2][2]=normals[3][2]=1.0f;

    /* Point 1 (Front) */
    texcoords[0][0]=1.0f; texcoords[0][1]=0.0f;
    vertices[0][0]=-1.0f; vertices[0][1]=-1.0f; vertices[0][2]=1.0f;
    /* Point 2 (Front) */
    texcoords[1][0]=0.0f; texcoords[1][1]=0.0f;
    vertices[1][0]=1.0f;  vertices[1][1]=-1.0f; vertices[1][2]=1.0f;
    /* Point 3 (Front) */
    texcoords[2][0]=0.0f; texcoords[2][1]=1.0f;
    vertices[2][0]=1.0f;  vertices[2][1]=1.0f; vertices[2][2]=1.0f;
    /* Point 4 (Front) */
    texcoords[3][0]=1.0f; texcoords[3][1]=1.0f;
    vertices[3][0]=-1.0f; vertices[3][1]=1.0f; vertices[3][2]=1.0f;

    /* Draw one textured plane using two stripped triangles */
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

    /* Back Face */
    /* Normal Pointing Away From Viewer */
    normals[0][0]=normals[1][0]=normals[2][0]=normals[3][0]=0.0f;
    normals[0][1]=normals[1][1]=normals[2][1]=normals[3][1]=0.0f;
    normals[0][2]=normals[1][2]=normals[2][2]=normals[3][2]=-1.0f;

    /* Point 1 (Back) */
    texcoords[0][0]=0.0f; texcoords[0][1]=0.0f;
    vertices[0][0]=-1.0f; vertices[0][1]=-1.0f; vertices[0][2]=-1.0f;
    /* Point 2 (Back) */
    texcoords[1][0]=0.0f; texcoords[1][1]=1.0f;
    vertices[1][0]=-1.0f; vertices[1][1]=1.0f; vertices[1][2]=-1.0f;
    /* Point 3 (Back) */
    texcoords[2][0]=1.0f; texcoords[2][1]=1.0f;
    vertices[2][0]=1.0f;  vertices[2][1]=1.0f; vertices[2][2]=-1.0f;
    /* Point 4 (Back) */
    texcoords[3][0]=1.0f; texcoords[3][1]=0.0f;
    vertices[3][0]=1.0f; vertices[3][1]=-1.0f; vertices[3][2]=-1.0f;

    /* Draw one textured plane using two stripped triangles */
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

    /* Top Face */
    /* Normal Pointing Up */
    normals[0][0]=normals[1][0]=normals[2][0]=normals[3][0]=0.0f;
    normals[0][1]=normals[1][1]=normals[2][1]=normals[3][1]=1.0f;
    normals[0][2]=normals[1][2]=normals[2][2]=normals[3][2]=0.0f;

    /* Point 1 (Top) */
    texcoords[0][0]=1.0f; texcoords[0][1]=1.0f;
    vertices[0][0]=-1.0f; vertices[0][1]=1.0f; vertices[0][2]=-1.0f;
    /* Point 2 (Top) */
    texcoords[1][0]=1.0f; texcoords[1][1]=0.0f;
    vertices[1][0]=-1.0f; vertices[1][1]=1.0f; vertices[1][2]=1.0f;
    /* Point 3 (Top) */
    texcoords[2][0]=0.0f; texcoords[2][1]=0.0f;
    vertices[2][0]=1.0f;  vertices[2][1]=1.0f; vertices[2][2]=1.0f;
    /* Point 4 (Top) */
    texcoords[3][0]=0.0f; texcoords[3][1]=1.0f;
    vertices[3][0]=1.0f;  vertices[3][1]=1.0f; vertices[3][2]=-1.0f;

    /* Draw one textured plane using two stripped triangles */
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

    /* Bottom Face */
    /* Normal Pointing Down */
    normals[0][0]=normals[1][0]=normals[2][0]=normals[3][0]=0.0f;
    normals[0][1]=normals[1][1]=normals[2][1]=normals[3][1]=-1.0f;
    normals[0][2]=normals[1][2]=normals[2][2]=normals[3][2]=0.0f;

    /* Point 1 (Bottom) */
    texcoords[0][0]=0.0f; texcoords[0][1]=1.0f;
    vertices[0][0]=-1.0f; vertices[0][1]=-1.0f; vertices[0][2]=-1.0f;
    /* Point 2 (Bottom) */
    texcoords[1][0]=1.0f; texcoords[1][1]=1.0f;
    vertices[1][0]=1.0f;  vertices[1][1]=-1.0f; vertices[1][2]=-1.0f;
    /* Point 3 (Bottom) */
    texcoords[2][0]=1.0f; texcoords[2][1]=0.0f;
    vertices[2][0]=1.0f;  vertices[2][1]=-1.0f; vertices[2][2]=1.0f;
    /* Point 4 (Bottom) */
    texcoords[3][0]=0.0f; texcoords[3][1]=0.0f;
    vertices[3][0]=-1.0f; vertices[3][1]=-1.0f; vertices[3][2]=1.0f;

    /* Draw one textured plane using two stripped triangles */
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

    /* Right face */
    /* Normal Pointing Right */
    normals[0][0]=normals[1][0]=normals[2][0]=normals[3][0]=1.0f;
    normals[0][1]=normals[1][1]=normals[2][1]=normals[3][1]=0.0f;
    normals[0][2]=normals[1][2]=normals[2][2]=normals[3][2]=0.0f;

    /* Point 1 (Right) */
    texcoords[0][0]=0.0f; texcoords[0][1]=0.0f;
    vertices[0][0]=1.0f;  vertices[0][1]=-1.0f; vertices[0][2]=-1.0f;
    /* Point 2 (Right) */
    texcoords[1][0]=0.0f; texcoords[1][1]=1.0f;
    vertices[1][0]=1.0f;  vertices[1][1]=1.0f; vertices[1][2]=-1.0f;
    /* Point 3 (Right) */
    texcoords[2][0]=1.0f; texcoords[2][1]=1.0f;
    vertices[2][0]=1.0f;  vertices[2][1]=1.0f; vertices[2][2]=1.0f;
    /* Point 4 (Right) */
    texcoords[3][0]=1.0f; texcoords[3][1]=0.0f;
    vertices[3][0]=1.0f;  vertices[3][1]=-1.0f; vertices[3][2]=1.0f;

    /* Draw one textured plane using two stripped triangles */
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

    /* Left Face*/
    /* Normal Pointing Left */
    normals[0][0]=normals[1][0]=normals[2][0]=normals[3][0]=-1.0f;
    normals[0][1]=normals[1][1]=normals[2][1]=normals[3][1]=0.0f;
    normals[0][2]=normals[1][2]=normals[2][2]=normals[3][2]=0.0f;

    /* Point 1 (Left) */
    texcoords[0][0]=1.0f; texcoords[0][1]=0.0f;
    vertices[0][0]=-1.0f; vertices[0][1]=-1.0f; vertices[0][2]=-1.0f;
    /* Point 2 (Left) */
    texcoords[1][0]=0.0f; texcoords[1][1]=0.0f;
    vertices[1][0]=-1.0f; vertices[1][1]=-1.0f; vertices[1][2]=1.0f;
      /* Point 3 (Left) */
    texcoords[2][0]=0.0f; texcoords[2][1]=1.0f;
    vertices[2][0]=-1.0f; vertices[2][1]=1.0f; vertices[2][2]=1.0f;
      /* Point 4 (Left) */
    texcoords[3][0]=1.0f; texcoords[3][1]=1.0f;
    vertices[3][0]=-1.0f; vertices[3][1]=1.0f; vertices[3][2]=-1.0f;

    /* Draw one textured plane using two stripped triangles */
    glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

    /* Disable texcoords and vertices arrays */
    glDisableClientState(GL_NORMAL_ARRAY);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);

    error=glGetError();
    if (error!=GL_NO_ERROR)
    {
       printf("OpenGL ES error: %08X\n", error);
    }

    glFinish();

    /* Gather our frames per second */
    Frames++;
    {
        GLint t=SDL_GetTicks();
        if (t-T0>=5000)
        {
            GLfloat seconds=(t-T0)/1000.0f;
            GLfloat fps = Frames/seconds;
            printf("%d frames in %g seconds = %g fps\n", Frames, seconds, fps);
            T0=t;
            Frames=0;
			loop++;
			if(loop > 5)
				return SDL_FALSE;
        }
    }

    xrot+=xspeed; /* Add xspeed To xrot */
    yrot+=yspeed; /* Add yspeed To yrot */

    return SDL_TRUE;
}

int CreateApp(int argc, char** argv)
{
    int status;

    /* main loop variable */
    SDL_bool done=SDL_FALSE;
    /* whether or not the window is active */
    SDL_bool isActive=SDL_TRUE;
    /* SDL event */
    SDL_Event event;

    status=SDL_Init(SDL_INIT_VIDEO);
    if (status<0)
    {
        fprintf(stderr, "Can't init default SDL video driver: %s\n", SDL_GetError());
        exit(-1);
    }

    /* Select first display */
    status=SDL_SelectVideoDisplay(0);
    if (status<0)
    {
        fprintf(stderr, "Can't attach to first display: %s\n", SDL_GetError());
        exit(-1);
    }

    /* Sets up OpenGL ES double buffering and visual parameters */
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
    SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 6);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);

    window=SDL_CreateWindow("Nehe: SDL/OpenGL ES Tutorial, Lesson 07",
        SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
        WINDOW_WIDTH, WINDOW_HEIGHT,
        SDL_WINDOW_RESIZABLE | SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN);
    if (window==0)
    {
        fprintf(stderr, "Can't create window: %s\n", SDL_GetError());
        DestroyApp(1);
    }

    glcontext=SDL_GL_CreateContext(window);
    if (glcontext==NULL)
    {
        fprintf(stderr, "Can't create OpenGL ES context: %s\n", SDL_GetError());
        DestroyApp(1);
    }

    status=SDL_GL_MakeCurrent(window, glcontext);
    if (status<0)
    {
        fprintf(stderr, "Can't set current OpenGL ES context: %s\n", SDL_GetError());
        DestroyApp(1);
    }

    /* Enable swap on VSYNC */
    SDL_GL_SetSwapInterval(1);

    /* initialize OpenGL */
    initGL();

    /* resize the initial window */
    resizeWindow(WINDOW_WIDTH, WINDOW_HEIGHT);

    /* wait for events */
    while (!done)
    {
        /* handle the events in the queue */
        while (SDL_PollEvent(&event))
        {
            switch (event.type)
            {
                case SDL_WINDOWEVENT:
                     switch (event.window.event)
                     {
                         case SDL_WINDOWEVENT_CLOSE:
                              done=SDL_TRUE;
                              break;
                         case SDL_WINDOWEVENT_HIDDEN:
                              isActive=SDL_FALSE;
                              break;
                         case SDL_WINDOWEVENT_SHOWN:
                              isActive=SDL_TRUE;
                              break;
                         case SDL_WINDOWEVENT_RESIZED:
                              resizeWindow(event.window.data1, event.window.data2);
                              break;
                     }
                     break;
                case SDL_KEYDOWN:
                     /* handle key presses */
                     handleKeyPress(&event.key.keysym, SDL_TRUE);
                     break;
                case SDL_KEYUP:
                     /* handle key presses */
                     handleKeyPress(&event.key.keysym, SDL_FALSE);
                     break;
                case SDL_QUIT:
                     /* handle quit requests */
                     done=SDL_TRUE;
                     break;
                default:
                     break;
            }
        }

        /* draw the scene */
        if (isActive==SDL_TRUE)
        {
           handleRepeatedKeys();
           if(!drawGLScene())
               done=SDL_TRUE;
           SDL_GL_SwapWindow(window);
        }
    }

    /* clean ourselves up and exit */
    DestroyApp(0);

    /* Should never get here */
    return 0;
}
